@@ -15,30 +15,42 @@ class KakiRequestHandler
 	DEFAULT_THEME_BASE_URL = './theme'
 
 	def initialize(page_data_source, variables, parameters)
-
-		@data = nil
-		@response_header = nil
-		@response_body = nil
-		@exception = nil
-		@site_property = nil
-		@wiki2html = nil
-		@request_command = nil
-		@request_page_id = nil
-		@request_content_source = nil
-		@request_edit_key = nil
-		@request_passwd = nil
-		@request_page_title = nil
-		@request_language_list = nil
-
-		@theme_base_url = nil
-		@theme_dir = nil
-		@template_dir = nil
-
+		self.reset();
 		self.page_data_source = page_data_source
 		self.request_variables = variables
 		self.request_parameters = parameters
 	end
 
+	def reset
+		@data = nil
+		@exception = nil
+		@site_property = nil
+		@page_data_source = nil
+		@wiki2html = nil
+
+		@request_variables = nil
+		@request_parameters = nil
+
+		@response_body = nil
+		@response_header = nil
+
+		@request_command = nil
+		@request_content_source = nil
+		@request_edit_key = nil
+		@request_language_list = nil
+		@request_page_id = nil
+		@request_page_title = nil
+		@request_passwd = nil
+
+		@script_path = nil
+		@server_uri = nil
+		@site_property = nil
+
+		@template_dir = nil
+		@theme_base_url = nil
+		@theme_dir = nil
+	end
+
 	attr_writer 'template_dir'
 	def template_dir
 		unless @template_dir then
@@ -188,14 +200,34 @@ protected
 		@site_property
 	end
 
+	def server_uri
+		unless @server_uri then
+			if request_variables['HTTPS'] then
+				scheme = "https" 
+			else
+				scheme = "http" 
+			end
+			host, port = request_variables['HTTP_HOST'].to_s.split(":", 2);
+			if port then
+				port = ":#{port}" 
+			else
+				port = "" 
+			end
+
+			@server_uri = "#{scheme}://#{host}#{port}"
+		end
+		@server_uri
+	end
+
 	def script_path
 		unless @script_path then
 			@script_path = self.request_variables['SCRIPT_NAME'].to_s
-			@script_path = '/' if script_path.length <= 0
+			@script_path = '/' if @script_path.length <= 0
 		end
 		@script_path
 	end
 
+
 	def data
 		unless @data then
 			@data = {}
@@ -665,11 +697,13 @@ protected
 				page_data[:page_title] = CGI.escapeHTML(page_property['page_title'])
 				page_data[:page_lang] = CGI.escapeHTML(lang)
 				page_data[:script_path] = self.script_path
+				page_data[:server_uri] = self.server_uri
 			end
 			page_data
 		}
 		page_list.delete(nil)
 		data[:page_list] = page_list
+		data[:server_uri] = self.server_uri
 
 		last_modified_time = self.site_property()['last_modified_time']
 		data.update(last_modified_time.to_data_hash)
@@ -1067,33 +1101,24 @@ protected
 		self.response_body = Misen.expand_text(Misen::STYLE_SGML, tmpl, self.data)
 	end
 
+
 	def append_referer(page_property)
 		referer = request_variables['HTTP_REFERER']
 		
 		if referer then
-			request_uri = URI.parse(request_variables['REQUEST_URI'])
-			if request_uri.host then
-				server_uri = request_uri.host
-				server_uri += ":#{request_uri.port}" if request_uri.port != 80
-			else
-				server_uri = request_variables['HTTP_HOST']
-			end
-			server_uri += request_uri.path
+			script_uri = server_uri + script_path
+			script_uri = URI.escape(URI.unescape(script_uri));
 
-			url_str = referer.to_s 
-			uri = URI.parse(url_str)
-			uri.fragment = nil
-			url_str = uri.to_s
-			referer_uri = uri.host
-			referer_uri += ":#{uri.port}" if uri.port != 80
-			referer_uri += uri.path
-			if server_uri != referer_uri then
+			referer_uri = URI.parse(URI.escape(URI.unescape(referer.to_s)))
+			referer_uri.fragment = nil
+			referer_uri_str = referer_uri.to_s
+			referer_uri.query = nil
+			referer_script_uri = referer_uri.to_s
+
+			if script_uri != referer_script_uri then
 				referers = page_property['referers'].to_s.split(", ")
-				referers.delete(url_str)
-				referers.delete_if() { |ref|
-					ref =~ /#{Regexp.escape(server_uri)}/ 
-				}
-				referers.unshift(url_str)
+				referers.delete(referer_uri_str)
+				referers.unshift(referer_uri_str)
 				referers = referers[0,10]
 				page_property['referers'] = referers.join(", ")
 			end
